/*****************************************************************
//
//      csparse.c
//
//      This file contains a simple interface to some of the
//      routines in the SPARSKIT library.  
//
//	For each function of the SPARSKIT library, a function (in C)
// 	which calls it is created. For example, function rperm_matrix
//	calls fortran function rperm_.
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 28, 1994
//
******************************************************************/

/*===========================================================================
			FUNCTIONS :
			----------

rperm_matrix :
transpose_matrix : 	calculates the transp of the matrix
multiply matrices : 	multiplies 2 sparse matrices
add_matrices : 		adds 2 sparse matrices
substract_matrices : 	substracts 2 sparse matrices
write_matrices : 	writes a sparse matrix in a Harwell-Boeing file
read_matrix : 		reads a matrix stored in a Harwell-Boeing file
gen57pt_matrix :	generates a 5-point and 7-point matrix
submat_matrix :		extracts a submatrix from a matrix
filter	:		filters the elements from a matrix according to 
			their magnitude
getdia : 		extracts a specified diagonal from a matrix.
getu :			extracts the upper triangular part of a matrix
getl :			extracts the lower triangular part of a matrix
getbwd :	   	gives the bandwith of the upper and the lower part
			of the mat
getbwdu :	   	gives the bandwith of the upper part of the matrix
getbwdl :	   	gives the bandwith of the lower part of the matrix
cnrms : 		computes the norms of the columns
roscal : 		scales the rows of a matrix by their norms
coscal : 		scales the columns of a matrix by their norms
csort  : 		sorts the elements in increasing order of columns
get_dim : 		gives the dimensions of a sparse matrix stored in a
			Harwell-Boeing file
indset0 :		greedy algorithm for independent set ordering
indset1 :		idem  but using minimal degree traversal
multic,colors,colors2 : greedy algorithm for multicoloring (the only difference
			between these functions is what they return)

FUNCTIONS ONLY USED BY SOLVE_SYSTEM (not available in spmath)

amux_matrix :		multilplies a matrix with a vector
atmux_matrix :		multiplies the transpose of a matrix with a vector
ilut_prec, iluk_prec : 	preconditionner
ilud_prec, ilu0_prec :  preconditionner
lusol_prec, lutsol_prec preconditionner
msrcsr_format :		create a matrix in compressed row format from a matrix
			in modified row format 
csrmsr_format :		create a matrix in modified row format from a matrix 
			in compressed row format
run*_solver :		these are 10 differents functions to run the solvers.

===========================================================================*/

#include "csparse.h"


void MAIN_ () {  
  /* This is just a dummy main to fool the FORTRAN compiler. */
}

void bandwidth (n, ja, ia, ml, mu, iband, bndav)
     int n;
     int *ja;
     int *ia;
     int *ml;
     int *mu;
     int *iband;
     double *bndav;
{
  bandwidth_ (&n, ja, ia, ml, mu, iband, bndav);
}

void nonz(n, sym, ja, ia, iao, nzmaxc, nzminc, nzmaxr, nzminr, nzcol, nzrow)
     int n;
     int sym;
     int *ja;
     int *ia;
     int *iao;
     int *nzmaxc;
     int *nzminc;
     int *nzmaxr;
     int *nzminr;
     int *nzcol;
     int *nzrow;
{
  nonz_ (&n, &sym, ja, ia, iao, nzmaxc, nzminc, nzmaxr, nzminr, nzcol, nzrow);
}

/*
void diag_dominance(n, sym, valued, a, ja, ia, ao, iao, ddomc, ddomr)
     int n;
     int sym;
     int valued;
     double *a;
     int *ja;
     int *ia;
     double *ao;
     int *iao;
     double *ddomc;
     double *ddomr;
{
  diag_domi_ (&n, &sym, &valued, a, ja, ia, ao, iao, ddomc, ddomr);
}
*/

void frobnorm(n, sym, a, ja, ia, Fnorm)
     int n;
     int sym;
     double *a;
     int *ja;
     int *ia;
     double *Fnorm;
{
  frobnorm_ (&n, &sym, a, ja, ia, Fnorm);
}

void ansym(n, sym, a, ja, ia, ao, jao, iao, imatch, av, fas, fan)
     int n;
     int sym;
     double *a;
     int *ja;
     int *ia;
     double *ao;
     int *jao;
     int *iao;
     int *imatch;
     double *av;
     double *fas;
     double *fan;
{
  ansym_ (&n, &sym, a, ja, ia, ao, jao, iao, imatch, av, fas, fan);
}

void distaij(n, nnz, sym, ja, ia, dist, std)
     int n;
     int nnz;
     int sym;
     int *ja;
     int *ia;
     double *dist;
     double *std;
{
  distaij_ (&n, &nnz, &sym, ja, ia, dist, std);
}

void skyline(n, sym, ja, ia, jao, iao, nsky)
     int n;
     int sym;
     int *ja;
     int *ia;
     int *jao;
     int *iao;
     int *nsky;
{
  skyline_ (&n, &sym, ja, ia, jao, iao, nsky);
}

void distdiag(nrow, ncol, ja, ia, dist)
     int nrow;
     int ncol;
     int *ja;
     int *ia;
     int *dist;
{
  distdiag_ (&nrow, &ncol, ja, ia, dist);
}

void bandpart(n, ja, ia, dist, nper, band)
     int n;
     int *ja;
     int *ia;
     int *dist;
     int nper;
     int *band;
{
  bandpart_ (&n, ja, ia, dist, &nper, band);
}

/*
void n_important_diag(n, nnz, dist, ipar1, ndiag, ioff, dcount)
     int n;
     int nnz;
     int *dist;
     int ipar1;
     int *ndiag;
     int *ioff;
     double *dcount;
{
  n_imp_diag_ (&n, &nnz, dist, &ipar1, ndiag, ioff, dcount);
}

void nonzero_lud(n, ja, ia, nlower, nupper, ndiag)
     int n;
     int *ja;
     int *ia;
     int *nlower;
     int *nupper;
     int *ndiag;
{
  printf("Bout to head over to nonz_lud\n");
  nonz_lud_ (&n, ja, ia, nlower, nupper, ndiag);
}

void avnz_column(n, ja, ia, iao, ndiag, av, st)
     int n;
     int *ja;
     int *ia;
     int *iao;
     int ndiag;
     double *av;
     double *st;
{
  avnz_col_ (&n, ja, ia, iao, &ndiag, av, st);
}
*/

void csrcsc(n, job, ipos, a, ja, ia, ao, jao, iao)
     int n;
     int job;
     int ipos;
     double *a;
     int *ja;
     int *ia;
     double *ao;
     int *jao;
     int *iao;
{
  csrcsc_ (&n, &job, &ipos, a, ja, ia, ao, jao, iao);
}

void rperm_matrix (nrow, a, ja, ia, ao, jao, iao, perm) 
  /* Calculate transp (a) (in place) */

  int nrow;            /* row dimension of a.                          */
  double *a;           /* The actual values of the matrix a.           */
  int *ja;             /* Column number of element a(i,j) in array a   */
  int *ia;             /* Pointer array for the sparse matrix a        */
  double *ao;          /* The actual values of the matrix ao.          */
  int *jao;            /* Column number of element ao(i,j) in array ao */
  int *iao;            /* Pointer array for the sparse matrix ao       */
  int *perm;	       /* Permutation array			       */

{
  int job = 1;	
  rperm_ (&nrow, a, ja, ia, ao, jao, iao, perm, &job);
} /* end rperm matrix */

void dperm_matrix (nrow, a, ja, ia, ao, jao, iao, perm, qperm, job) 
  /* Calculate transp (a) (in place) */

  int nrow;            /* row dimension of a.                          */
  double *a;           /* The actual values of the matrix a.           */
  int *ja;             /* Column number of element a(i,j) in array a   */
  int *ia;             /* Pointer array for the sparse matrix a        */
  double *ao;          /* The actual values of the matrix ao.          */
  int *jao;            /* Column number of element ao(i,j) in array ao */
  int *iao;            /* Pointer array for the sparse matrix ao       */
  int *perm;	       /* Permutation array (rows)		       */
  int *qperm;		/* Permutation array (columns)			*/
  int job;		/* job indicator				*/

{
  	
  dperm_ (&nrow, a, ja, ia, ao, jao, iao, perm, qperm, &job);
} /* end dperm matrix */

void cperm_matrix (nrow, a, ja, ia, ao, jao, iao, perm) 
  /* Calculate transp (a) (in place) */

  int nrow;            /* row dimension of a.                          */
  double *a;           /* The actual values of the matrix a.           */
  int *ja;             /* Column number of element a(i,j) in array a   */
  int *ia;             /* Pointer array for the sparse matrix a        */
  double *ao;          /* The actual values of the matrix ao.          */
  int *jao;            /* Column number of element ao(i,j) in array ao */
  int *iao;            /* Pointer array for the sparse matrix ao       */
  int *perm;	       /* Permutation array			       */

{
  int job = 1;	
  cperm_ (&nrow, a, ja, ia, ao, jao, iao, perm, &job);
} /* end cperm matrix */


void transpose_matrix (nrow, ncol, a, ja, ia, iwk) 
  /* Calculate transp (a) (in place) */

  int ncol;            /* column dimension of a.                     */
  int nrow;            /* row dimension of a.                        */
  double *a;           /* The actual values of the matrix a.         */
  int *ja;             /* Column number of element a(i,j) in array a */
  int *ia;             /* Pointer array for the sparse matrix a      */
  int *iwk;            /* Work array.                                */

{

  int ierr;            /* Integer to hold any errors that may occur.  */

  transp_ (&nrow, &ncol, a, ja, ia, iwk, &ierr);
  assert (ierr == 0);
} /* end transpose matrix */

void multiply_matrices (nrow, ncol, a, ja, ia, b, jb, ib, c, jc, ic, nzmax, iw)
     /* Calculate c = a * b */

  int ncol;            /* column dimension of a.                     */
  int nrow;            /* row dimension of a.                        */
  double *a;           /* The actual values of the matrix a.         */
  int *ja;             /* Column number of element a(i,j) in array a */
  int *ia;             /* Pointer array for the sparse matrix a      */
  double *b;           /* The actual values of the matrix b.         */
  int *jb;             /* Column number of element a(i,j) in array b */
  int *ib;             /* Pointer array for the sparse matrix b      */
  double *c;           /* The actual values of the matrix c.         */
  int *jc;             /* Column number of element a(i,j) in array c */
  int *ic;             /* Pointer array for the sparse matrix c      */
  int nzmax;           /* Maximum number of elements that c can hold */
  int *iw;             /* Work array.                                */

{

  int ierr;            /* Integer to hold any errors that may occur.  */
  int job = 1;         /* Job to indicate that the real values are to */
                       /* be considered.                              */

  amub_ (&nrow, &ncol, &job, a, ja, ia, b, jb, ib, c, jc, ic, &nzmax, iw, &ierr);

  assert (ierr == 0);

} /* end multiply_matrices. */

void add_matrices (nrow, ncol, a, ja, ia, b, jb, ib, c, jc, ic, nzmax, iw) 
  /* Calculate c = a + c                                             */

  int ncol;            /* column dimension of a, b, c.               */
  int nrow;            /* row dimension of a, b, c.                  */
  double *a;           /* The actual values of the matrix a.         */
  int *ja;             /* Column number of element a(i,j) in array a */
  int *ia;             /* Pointer array for the sparse matrix a      */
  double *b;           /* The actual values of the matrix b.         */
  int *jb;             /* Column number of element a(i,j) in array b */
  int *ib;             /* Pointer array for the sparse matrix b      */
  double *c;           /* The actual values of the matrix c.         */
  int *jc;             /* Column number of element a(i,j) in array c */
  int *ic;             /* Pointer array for the sparse matrix c      */
  int nzmax;           /* Maximum number of elements that c can hold */
  int *iw;             /* Work array.                                */

{

  int ierr;            /* Holds any errors that might have occured.  */
  int job = 1;         /* Job to indicate that the values should be  */
                       /* added along with the structure.            */


  aplb_ (&nrow, &ncol, &job, a, ja, ia, b, jb, ib,
	 c, jc, ic, &nzmax, iw, &ierr);

  assert (ierr == 0);

} /* end add_matrices */

void subtract_matrices (nrow, ncol, a, ja, ia, b, jb, ib, c, jc, ic, nzmax) 
  /* Calculate c = a - b                                             */

  int ncol;            /* column dimension of a, b, c.               */
  int nrow;            /* row dimension of a, b, c.                  */
  double *a;           /* The actual values of the matrix a.         */
  int *ja;             /* Column number of element a(i,j) in array a */
  int *ia;             /* Pointer array for the sparse matrix a      */
  double *b;           /* The actual values of the matrix b.         */
  int *jb;             /* Column number of element a(i,j) in array b */
  int *ib;             /* Pointer array for the sparse matrix b      */
  double *c;           /* The actual values of the matrix c.         */
  int *jc;             /* Column number of element a(i,j) in array c */
  int *ic;             /* Pointer array for the sparse matrix c      */
  int nzmax;           /* Maximum number of elements that c can hold */

{

  int ierr;            /* Holds any errors that might have occured.  */
  double s = -1;       /* Scaler to multiply b by. since we want     */
                       /* simple subtraction, s = -1                 */

  aplsb_ (&nrow, &ncol, a, ja, ia, &s, b, jb, ib,
	 c, jc, ic, &nzmax, &ierr);

  assert (ierr == 0);

} /* end subtract_matrices */


void write_matrix (filename, nrow, ncol, nnz, a, ja, ia, title, key, type, ifmt)
  /* Write the matrix to the file filename.                          */

  char *filename;      /* The filename to be read from.              */
  int nrow;            /* The number of rows in the matrix.          */
  int ncol;            /* The number of columns in the matrix.       */
  int nnz;             /* The number of non zero elements in the mat.*/
  double *a;           /* The actual values of the matrix.           */
  int *ja;             /* Column number of element a(i,j) in array a */
  int *ia;             /* Pointer array for the sparse matrix.       */
  char * title;        /* Hold the title of the matrix.              */
  char * key;          /* Hold the key of the matrix.                */
  char * type;         /* Hold the type of the matrix.               */
  int ifmt;            /* Integer to specify the format for the real */
                       /* values. See the documentation for the      */
                       /* Fortran routine prtmt for a complete       */
                       /* description.                               */

{

  char guesol[2];      /* Passed to readmt_ but not used                */
  double  *rhs = NULL; /* Passed to readmt_ but not used.               */
  int job = 2;         /* The job specifying that the structur and the  */
                       /* values are to be written.                     */

  prtmt_ (&nrow, &ncol, a, ja, ia, rhs, guesol, title, 
	  key, type, &ifmt, &job, filename); 

} /* end write_matrix */

void submat_matrix(n, job, i1, i2, j1, j2, a, ja, ia, nr, nc, a0, ja0, ia0)

	int n;		/* row dimension of the matrix */
	int job;	/* job indicator (see formats/unary.f)*/
	int i1;		
	int i2;		/* i1 and i2 : range of the rows to be extracted*/
	int j1;		
	int j2;		/* j1, j2 : range of the columns to be extracted */
	double *a;	
	int *ja;
	int *ia;	/* a, ja, ia: matrix in compressed row format */
	int *nr;	/* number of rows of the submatrix */
	int *nc;	/* number of the columns of the submatrix */
	double *a0;
	int *ja0;
	int *ia0;	/* extracted matrix in sparse row format */

{
	submat_ (&n,&job,&i1,&i2,&j1,&j2,a,ja,ia,nr,nc,a0,ja0,ia0);

}

void filter_matrix(n,job, drptol, a, ja, ia, b, jb, ib, len, ierr)

	int n;		/* row dimension of the matrix */
	int job;	/* job indicator (see formats/unary.f) */
	double drptol;	/* drop tolerance used for drooping strategy */
	double *a;	
	int *ja;
	int *ia;	/* a, ja, ia: input matrix in sparse row format */	
	double *b;
	int *jb;
	int *ib;	/* b, jb, ib: resulting matrix in sparse row format */
	int len;	/* amount of space available in arrays b and jb */
	int ierr;	/* error message */

{
	filter_(&n, &job, &drptol, a, ja, ia, b, jb, ib, &len, &ierr);
}

void csort_matrix (n, a, ja, ia, iwork, values)

	int n;		/* row dimension of the matrix */
	double *a;
	int *ja;
	int *ia;	/* a, ja, ia: matrix stored in sparse row format */
	int *iwork;	/* work array */
	int values;	/* job indicator (see genmat) */	

{
	csort_ (&n, a, ja, ia, iwork, &values);
}

void getdia_matrix (nrow, ncol, job, a, ja, ia, len, diag, idiag, ioff )

	int nrow;	/* row dimension of the matrix */
	int ncol;	/* col dimension of the matrix */
	int job;	/* job indicator (0-> give the diag, else remove it) */
	double *a;	
	int *ja;
	int *ia;	/* matrix in compressed row format */
	int *len;	/* number of non zero elements of the diag */
	double *diag;	/* elements of the diag */
	int *idiag;	/* positions of the non zero elements of the diag */
	int ioff;	/* number of the diag */


{
	getdia_(&nrow, &ncol, &job, a, ja, ia, len, diag, idiag, &ioff);
}

void getu_matrix(n, a, ja, ia, a0, ja0, ia0)

	int n;		/*  dimension of the matrix*/
	double *a;
	int *ja;
	int *ia;	/* a, ja, ia: matrix in compressed row format*/
	double *a0;
	int *ja0;
	int *ia0;	/* resulting matrix in compressed row format */

{	getu_ (&n, a, ja, ia, a0, ja0, ia0);	}

void getl_matrix(n, a, ja, ia, a0, ja0, ia0)

	int n;		/*  dimension of the matrix*/
	double *a;
	int *ja;
	int *ia;	/* a, ja, ia: matrix in compressed row format*/
	double *a0;
	int *ja0;
	int *ia0;	/* resulting matrix in compressed row format */


{	getl_ (&n, a, ja, ia, a0, ja0, ia0);	}

void blkfnd_matrix (nrow, ja, ia, nblk )

	int nrow;	/*row dimension of the matrix		*/
	int *ja;
	int *ia;	/* matrix */
	int *nblk;	/* result: block size		*/

{	blkfnd_ (&nrow, ja, ia, nblk);	}

void cnrms_matrix (nrow, nrm, a, ja, ia, diag)

	int nrow;
	int nrm;
	double *a;
	int *ja;
	int *ia;
	double *diag;

{	cnrms_ (&nrow, &nrm, a, ja, ia, diag);	}

void roscal_matrix(nrow, job, nrm, a, ja, ia, diag, b, jb, ib, ierr)

	int nrow;
	int job;
	int nrm;
	double *a;
	int *ja;
	int *ia;
	double *diag;
	double *b;
	int *jb;
	int *ib;
	int ierr;

{	roscal_ (&nrow, &job, &nrm, a, ja, ia, diag, b, jb, ib, &ierr);	}

void coscal_matrix(nrow, job, nrm, a, ja, ia, diag, b, jb, ib, ierr)

	int nrow;
	int job;
	int nrm;
	double *a;
	int *ja;
	int *ia;
	double *diag;
	double *b;
	int *jb;
	int *ib;
	int *ierr;

{	coscal_ (&nrow, &job, &nrm, a, ja, ia, diag, b, jb, ib, ierr);	}

void addblk_matrices (nrowa, ncola, a, ja, ia, ipos, jpos, nrowb, ncolb, b, jb, ib, nrowc, ncolc, c, jc, ic, nzmax, ierr)

	int nrowa;
	int ncola;
	double *a;
	int *ja;
	int *ia;
	int ipos;
	int jpos;
	int nrowb;
	int ncolb;
	double *b;
	int *jb;
	int *ib;
	int *nrowc;
	int *ncolc;
	double *c;
	int *jc;
	int *ic;
	int nzmax;
	int ierr;

{	int job=1;	
	addblk_(&nrowa, &ncola, a, ja, ia, &ipos, &jpos, &job, &nrowb, &ncolb, b, jb, ib, &nrowc, &ncolc, c, jc, ic, &nzmax, &ierr);	}

void get1up_matrix(n, ja, ia, ju)

	int n;
	int *ja;
	int *ia;
	int *ju;

{	get1up_(&n, ja, ia, ju);		}

void infdia_matrix(n, ja, ia, ind, idiag)

	int n;
	int *ja;
	int *ia;
	int *ind;
	int *idiag;

{	infdia_(n, ja, ia, ind, idiag);	}

void gen57pt_matrix(nx, ny, nz, al, mode, n, a, ja, ia, iau, rhs)

	int nx;		/* number of grid points in x direction */
	int ny;		/* number of grid points in y direction */
	int nz;		/* number of grid points in z direction */
	double *al;	/* coefficient alpha of the boundaries conditions */
	int mode;	/* to decide what to generate (graph, matrix, rhs??) */
	int *n;		/* number of rows of the matrix */ 
	double *a;	/* The values of the matrix */
	int *ja;	/* Column number of element a(i,j) in array a */
	int *ia;	/* Pointer array for the sparse matrix */
	int *iau;	/* position of the diagonal element of the matrix*/
	double *rhs;	/* right hand side */

{
	gen57pt_(&nx, &ny, &nz, al, &mode, n, a, ja, ia, iau, rhs);

}




void read_matrix (filename, nrow, ncol, nnz, a, ja, ia, title, key, type)
  /* This routine reads in a sparse matrix from the file filename.   */

  char *filename;      /* The filename to be read from.              */
  int nrow;            /* The number of rows in the matrix.          */
  int ncol;            /* The number of columns in the matrix.       */
  int nnz;             /* The number of non zero elements in the mat.*/
  double *a;           /* The actual values of the matrix.           */
  int *ja;             /* Column number of element a(i,j) in array a */
  int *ia;             /* Pointer array for the sparse matrix.       */
  char * title;        /* Hold the title of the matrix.              */
  char * key;          /* Hold the key of the matrix.                */
  char * type;         /* Hold the type of the matrix.               */

{
 

  int  nmax, nzmax;    /* Used by readmt_                               */
  int job = 2;         /* The job to determine what is done in readmt_  */
  int nrhs;            /* Passed to readmt_ but not used                */
  char guesol[2];      /* Passed to readmt_ but not used                */
  double  *rhs = NULL; /* Passed to readmt_ but not used.               */
  int ierr;            /* Passed to readmt_ to check for errors.        */
  /* Change by Ben langmead - next two variables commented out */
  /* int int_dummy; */      /* Dummy variable to determine the size of int.  */
  /*double double_dummy; */ /* Dummy variable to determine the size of double*/

  /* Set nmax and nzmax to the dimensions of ia and ja respectively.    */
  nmax = ncol + 1;   /*  I don't know if the +1 is necesarry or not     */
  nzmax = nnz + 1;   /*  Ditto                                          */
  
  /* Read the  matrix from the file.                                    */
  readmt_ (&nmax, &nzmax, &job, filename, a, ja, ia, rhs, &nrhs, 
	   guesol, &nrow, &ncol, &nnz, title, key, type, &ierr);

  assert (ierr == 0);

  /* Since fortran does not put null characters at the end of their     */
  /* strings, do it manually.  This is why  the strings had to be a     */
  /* little bit longer. (1 more each).                                  */
  title[71] = '\0';
  key[8] = '\0';
  type[3] = '\0';

} /* end read_matrix */


void get_dim (filename, nrow, ncol, nnz) 
  /* This routine gets the dimensions of the sparse matrix stored in */
  /* filename.  It returns the number of rows, columns, and non zero */
  /* elements.                                                       */

  char *filename;      /* The filename to be read from.              */
  int *nrow;           /* Return the number of rows.                 */
  int *ncol;           /* Return the number of  columns.             */
  int *nnz;            /* Return the number of non zero elements.    */

{

  int  nmax, nzmax;    /* Used by readmt_                               */
  int job = 0;         /* The job to determine what is done in readmt_  */
  int nrhs = 0;        /* Passed to readmt_ but not used                */
  char guesol[2];      /* Passed to readmt_ but not used                */
  double  *rhs = NULL; /* Passed to readmt_ but not used.               */
  int ierr = 0;        /* Passed to readmt_ to check for errors.        */
  char title[72];      /* To hold the title, but is not returned.       */
  char key[8];         /* To hold the  key, but is not returned.        */
  char type[3];        /* To hold the type, but is not returned.        */
  int *ja = NULL;      /* Dummy argument for readmt_                    */
  int *ia = NULL;      /* Dummy argument for readmt_                    */
  double *a = NULL;    /* Dummy argument for readmt_                    */

  *nrow = 0;           /* Assign to zero to avoid errors.               */
  *ncol = 0;           /* Assign to zero to avoid errors.               */
  *nnz = 0;            /* Assign to zero to avoid errors.               */

  readmt_ (&nmax, &nzmax, &job, filename, a, ja, ia, rhs, &nrhs, 
	   guesol, nrow, ncol, nnz, title, key, type, &ierr);

  assert (ierr == 0);

} /* end get_dim */


void getbwd_matrix(n, a, ja, ia, ml, mu)

  	int n;	       /* row dimension of the matrix	*/
	double *a;
	int *ja;
	int *ia;       /* a, ja, ia: matrix stored in compressed row format */
	int *ml;       /* bandwith of the lower triangular part of the matrix*/
	int *mu;       /* bandwith of the upper triang. part of the matrix */

{	getbwd_ (&n, a, ja, ia, ml, mu);		}

void amux_matrix (n, x, y, a, ja, ia)

	int n;		/* row dimension of the matrix */
	double *x;	
	double *y;
	double *a;
	int *ja;
	int *ia;

{	amux_ (&n, x, y, a, ja, ia);		}

void atmux_matrix (n, x, y, a, ja, ia)

	int n;
	double *x;
	double *y;
	double *a;
	int *ja;
	int *ia;

{	atmux_ (&n, x, y, a, ja, ia);		}

void ilut_prec(n, a, ja, ia, lfil, drptol, alu, jlu, ju, iwk, w, jw, ierr)

	int n;
	double *a;
	int *ja;
	int *ia;
	int lfil;
	double drptol;
	double *alu;
	int *jlu;
	int *ju;
	int iwk;
	double *w;
	int *jw;
	int *ierr;

{
	ilut_ (&n, a, ja, ia, &lfil, &drptol, alu, jlu, ju, &iwk, w, jw, ierr);
}

void ilutp_prec(n, a, ja, ia, lfil, drptol, permtol, mbloc, alu, jlu, ju, iwk, w, jw, iperm, ierr)

	int n;
	double *a;
	int *ja;
	int *ia;
	int lfil;
	double drptol;
        double permtol;
	int mbloc;
	double *alu;
	int *jlu;
	int *ju;
	int iwk;
	double *w;
	int *jw;
	int *iperm;
	int *ierr;

{
	ilutp_ (&n, a, ja, ia, &lfil, &drptol, &permtol, &mbloc, alu, 
		     jlu, ju, &iwk, w, jw, iperm, ierr);
}

void iluk_prec(n, a, ja, ia, lfil, alu, jlu, ju, levs, iwk, w, jw, ierr)

int n;
double *a;
int *ja;
int *ia;
int lfil;
double *alu;
int *jlu;
int *ju; 
int *levs;
int iwk;
double *w;
int *jw;
int *ierr;

{	iluk_ (&n, a, ja, ia, &lfil, alu, jlu, ju, levs, &iwk, w, jw, ierr);	}

void ilu0_prec(n, a, ja, ia, alu, jlu, ju, iw, ierr)

int n;
double *a;
int *ja;
int *ia;
double *alu; 
int *jlu;
int *ju;
int *iw;
int *ierr;

{	ilu0_ (&n, a, ja, ia, alu, jlu, ju, iw, ierr);		}

void ilud_prec(n, a, ja, ia, alph, tol, alu, jlu, ju, iwk, w, jw, ierr)

	int n;
	double *a;
	int *ja;
	int *ia;
	double alph;
	double tol;
	double *alu;
	int *jlu;
	int *ju;
	int iwk;
	double *w;
	int *jw;
	int *ierr;

{
	ilud_ (&n, a, ja, ia, &alph, &tol, alu, jlu, ju, &iwk, w, jw, ierr);
}

void lusol_prec(n, y, x, alu, jlu, ju)

	int n;
	double *y;
	double *x;
	double *alu;
	int *jlu;
	int *ju;

{	lusol_(&n, y, x, alu, jlu, ju);		}

void lutsol_prec(n, y, x, alu, jlu, ju)

	int n;
	double *y;
	double *x;
	double *alu;
	int *jlu;
	int *ju;

{	lutsol_(&n, y, x, alu, jlu, ju);		}


void msrcsr_format (n, a, ja, a0, ja0, ia0, wk, iwk)

	int n;
	double *a;
	int *ja;
	double *a0; 
	int *ja0;
	int *ia0;
	double *wk;
	int *iwk;

{	msrcsr_(&n, a, ja, a0, ja0, ia0, wk, iwk);		}

void csrmsr_format(n, a, ja, ia, a0, ja0, wk, iwk)

	int n;
	double *a;
	int *ja;
	int *ia;
	double *a0;
	int *ja0;
	double *wk;
	int *iwk;

{	csrmsr_(&n, a, ja, ia, a0, ja0, wk, iwk);	}

void rungmres_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	
  runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,gmres_);
}

void runbcg_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,bcg_);	}

void runfgmres_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,fgmres_);		}

void runcg_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,cg_);		}

void rundbcg_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,dbcg_);		}

void runcgnr_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,cgnr_);		}

void runbcgstab_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau, bcgstab_);		}

void runtfqmr_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,tfqmr_);		}

void runfom_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,fom_);		}

void rundqgmres_solver(n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau)

int n;
double *rhs;
double *sol;
int *ipar;
double *fpar;
double *wk;
double *guess;
double *a;
int *ja;
int *ia;
double *au;
int *jau;
int *iau;

{	runrc_ (&n, rhs, sol, ipar, fpar, wk, guess, a, ja, ia, au, jau, iau,dqgmres_);		}

void multic_matrix (n, ja, ia, ncol, kolrs, il, iord, maxcol, ierr)

int n;
int *ja; 
int *ia;
int *ncol;
int *kolrs;
int *il;
int *iord;
int maxcol;
int *ierr;

{	multic_ (&n, ja, ia, ncol, kolrs, il, iord, &maxcol, ierr);		}

void indset0_matrix (n, ja, ia, nset, iord, riord, sym, iptr)

int n;
int *ja;
int *ia;
int *nset;
int *iord;
int *riord;
int iptr;
int sym;

{	indset0_(&n, ja, ia, nset, iord, riord, &iptr, &sym);	}

void indset1_matrix (n, ja, ia, nset, iord, riord, iw, sym, iptr)

int n;
int *ja;
int *ia;
int *nset;
int *iord;
int *riord;
int *iw;
int iptr;
int sym;

{	indset1_(&n, ja, ia, nset, iord, riord, iw, &iptr, &sym);	}
