//****************************************************************
//
//    Class funct_def
//
//    Description:
//      Funct_def is a class for storing information of a function
//	definition.
// 
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
//****************************************************************

#include "funct_def.h"

/*===========================================================================
				PRIVATE :
				--------

    char * ret;     // name of the value returned by the function
    char * args;    // comma-list of the arguments
    char * body;    // body of the function

	       		       FUNCTIONS :
			       ----------
constructors
destructor

get_ret : 	returns the name of the variable returned by the function
get_args : 	returns the list of the arguments 
get_body : 	returns the body of the function
get_argc : 	returns the number of the arguments of the function
get_arg(i) : 	returns a string pointer to the ith argument of the function

===========================================================================*/



//****************************************************************
//	Constructors
//****************************************************************

funct_def::funct_def() {
  // Create an empty function

} // end constructor funct_def

funct_def::funct_def (char *i, char *j, char *k) {
  // Create a function definition

  ret = i;
  args = j;
  body = k;

} // end constructor funct_def

funct_def::funct_def (const funct_def & f) {
  // Copy constructor

  ret = f.get_ret();
  args = f.get_args();
  body = f.get_body();

} // end constructor funct_def

void funct_def::free() {
  // Dealocate strings.

  if(ret)  { delete [] ret;  ret = NULL;  }
  if(args) { delete [] args; args = NULL; }
  if(body) { delete [] body; body = NULL; }

} // end free

funct_def::~funct_def() {
  //  if(ret)  delete[] ret;
  //  if(args) delete[] args;
  //  if(body) delete[] body;
}

//****************************************************************
//	Selectors
//****************************************************************

/*=========================================================================
#			GET_RET :
#			--------
# Entries : none
# Returns : the name of the variable returned by the function 
#
=========================================================================*/

char * funct_def::get_ret() const {
  return ret;
}

/*=========================================================================
#			GET_ARGS :
#			---------
# Entries : none
# Returns : the comma-list of the arguments of the function
#
=========================================================================*/

char * funct_def::get_args() const {
  return args;
}

/*=========================================================================
#			GET_BODY :
#			--------
# Entries : none
# Returns : the body of the function
#
=========================================================================*/

char * funct_def::get_body() const {
  return body;
}

/*=========================================================================
#			GET_ARGC :
#			---------
# Entries : none
#
# Returns : the number of the arguments of the function
#
# Details :  - It counts the number of comma in the attribute args
#	     - If this number is not equal to 0, it returns this number
#	     - else it decides if there is 0 or 1 arguments :
#		  + if there is a char different from '(', ')', ' ' or '\t'
#		    it means that there is one argument 
#		  + else there is no arguments
#
=========================================================================*/

int funct_def::get_argc() const {
  // Return the number of arguments in the string args.

  int i;
  int len = strlen (args);
  int argc = 0;

  for (i = 0; i  <= len - 1; i += 1) 
    if (args[i] == ',')
      argc += 1;

  if (argc > 0)
    return argc + 1;

  // Otherwise, not sure if there is one or zero arguments.
  for (i = 0; i  <= len - 1; i += 1) 
    if (args[i] != '(' && args[i] != ')' && args[i] != ' ' && args[i] != '\t')
      return 1;

  // Must have zero arguments
  return 0;

} // end selector get_argc

/*=========================================================================
#			GET_ARG :
#			--------
# Entries : an integer : i 
#
# Returns : a string pointer to the i'th argument
#
# Details :   - it checks if i is inferior to the number of args of the function
#	      - it checks if i > 0
#	      - it returns the string which is between the (i-1)'th and the i'th
#	        comma
#
=========================================================================*/

char * funct_def::get_arg (int i) const {
  // Return a string pointer to the i'th argument

  assert (i > 0);
  assert (i <= get_argc());

  int count = 0;
  int j = 0;

  while (count < i) {
    if (args[j] == ',' || args[j] == '(')
      count += 1;
    j += 1;
  } // end while

  int k = j;

  // Count how long the i'th argument is
  while (args[k] != ',' && args[k] != ')')
    k += 1;

  char *val = new char [k - j+ 1];
  k = j;
  int index = k - j;

  // Copy the i'th arg into val
  while (args[k] != ',' && args[k] != ')') {
    if (is_letter (args[k]))
      val[index++] = args[k];
    k += 1;
  } // end while
  val [index] = '\0';

  return val;

} // end get_arg


//***************************************************************
// 	Functions
//***************************************************************

int funct_def::size() {
  return (strlen(ret) + 
	  strlen(body) + 
	  strlen(args) +
	  sizeof(*this));
}
