//****************************************************************
//
//      Class string
//
//      String is a class for implementing an abstract data type
//      for  character strings.  It is realy just a simpler 
//      interface to the C string library.
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 11, 1994
//
//****************************************************************

/*==========================================================================
				PRIVATE : 
				--------

    char *ptr;       // pointer to the first character of the string

				FUNCTIONS :
				----------

constructors
destructor

empty :			returns 1 if the string is NULL
get_string(s) : 	copies the value of the string into s
get_string(a,b) : 	returns the string with from the index a to b
length : 		returns the lenght of the string
get_char(i) : 		returns the ith char of the string
set_char(c,i) : 	sets the ith char to c
remove_leading_spaces : removes the spaces from the string up to the first
			character that is not a space or the NULL character
remove_trailing_spaces :removes the spaces from the end of the string up to the
			last character that is not a space

operators : 		=, <<, [], +, <, ==, >, !=, ()

is_letter(c) : 		returns 1 if c is an upper or lower case letter
is_numeral(c) : 	returns 1 if c is numeral
input(st) : 		gets a string from the keyboard (finished by a double
			return)

==========================================================================*/

#ifndef STRING_H
#define STRING_H

#include <iostream.h>
#include <fstream.h>
#include <assert.h>
#include <string.h>


class string {
  
  public:
     
    // constructors
    string ();
    string (const char *);
    string (const string &);

    // destructors
    ~string ();
  
    // selectors
    int    empty () const;
    char * get_pointer () const;
    void   get_string (char *) const;
    string get_string (int, int) const;
    int    length () const;
    char   get_char (int) const;

    int size();

    // member functions 
    void remove_leading_spaces();
    void remove_trailing_spaces();    
    void set_char (char, int);

    // operators
    string operator = (const string &);    
    string operator <<= (string &);    
    char& operator [] (int);
    operator char * ();

  private:

    // data fields
    char *ptr;       // pointer to the first character of the string
};

string operator +  (const string &, const string &);
int operator <  (const string &, const string &);
int operator == (const string &, const string &);
int operator >  (const string &, const string &);
int operator != (const string &, const string &);

void output (const string &);
int is_letter (char);
int is_numeral (char);
string remove_leading_spaces (string &);
string remove_trailing_spaces (string &);
string input (string &);
#endif
