// ****************************************************************
//
//    Class value
//
//    Description:
//      Value is a class for subverting C++ strong typing.
//      It does this by having a union  of pointers and then
//      an integer flag to keep track of the type (and pointer)
//      that is being used.
//      A value can be either a char*, a double,  a dense matrix, a sparse
//	matrix or a function.
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
// ****************************************************************

/*===========================================================================
				PRIVATE :
				--------
    int type;			// type of the value
    union { 			// value 
      char *str;
      double num;
      dense *dmat;
      sparse *smat;
      funct_def *funct;

These attributes are public

				FUNCTIONS :
				----------

constructors
destructor

operator =
output : outputs the value (doesn't print the type )   

===========================================================================*/

#include "spmath.h"

//******************************************************************
//       Member Constructors
//******************************************************************
 
value::value () {
  type = NO_TYPE;
}

value::value (char *n) {
  type = STRING;
  str = n;
} // end constructor value

value::value (double n) {
  type = NUM;
  num = n;
} // end constructor value

value::value (int n) {
  type = NUM;
  num = n;
} // end constructor value

value::value (dense *d) {
  type = DENSE;
  dmat = d;
} // end constructor value

value::value (sparse *s) {
  type = SPARSE;
  smat = s;
} // end constructor value

value::value (funct_def *f) {
  type = FUNCT;
  funct = f;
} // end constructor value

value::value (const value &x) {
  type = x.type;
  if (type == NUM)
    num = x.num;
  else if (type == SPARSE)
    smat = x.smat;
  else if (type == DENSE)
    dmat = x.dmat;
  else if (type == STRING)
    str = x.str;
  else if (type == FUNCT)
    funct = x.funct;
  
} // end constructor value

//******************************************************************
//       Member Mutators
//******************************************************************

void value::free() {
  // Free up dynamically allocated memory

  if (type == STRING) 
    delete [] str;
  if (type == NUM) 
    ; // do nothing
  if (type == DENSE) {
    dmat->free();
    delete dmat;
  }
  if (type == SPARSE) {
    smat->free();
    delete smat;
  }
  if (type == FUNCT) {
    funct->free();
    delete funct;
  }
} // end destructor free

//******************************************************************
//       Member Operators
//******************************************************************

value value::operator = (const value &x) {
  // Assignement operator

  type = x.type;
  if (type == NUM)
    num = x.num;
  if (type == SPARSE)
    smat = x.smat;
  if (type == DENSE)
    dmat = x.dmat;
  if (type == STRING)
    str = x.str;
  if (type == FUNCT)
    funct = x.funct;
  return x;

} // end operator =

char * value::get_str() {

  if (type == STRING) return (str);
  return strdup("non");
}

double value::get_num() {

  if (type == NUM) return num;
  return (-1);
}

//******************************************************************
//       Functions
//******************************************************************

void output (value *val) {
  // Output the value.

  if (val->type == STRING) {
    sp_out(val->str);
  }
  else if (val->type == NUM) {
    sp_out(val->num);
  }
  else if (val->type == DENSE) { 
    //    if (val->dmat->get_nrow() <= 200)
    //      if (val->dmat->get_ncol() <= 200)
    
    // Make user supress output if they want to

    output (*val->dmat);
  }

  if (val->type == SPARSE) {
    //    if (val->smat->get_nrow() <= 1000)
    //      if (val->smat->get_ncol() <= 1000)

    // Make user supress output if they want to

    output (*val->smat);
  }
  // No output for  functions

} // end output

