// ****************************************************************
//
//    Class value
//
//    Description:
//      Value is a class for subverting C++ strong typing.
//      It does this by having a union  of pointers and then
//      an integer flag to keep track of the type (and pointer)
//      that is being used.
//      A value can be either a char*, a double,
//      a dense matrix, a sparse matrix or a function.
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
// ****************************************************************

/*===========================================================================
				PRIVATE :
				--------
    int type;			// type of the value
    union { 			// value 
      char *str;
      double num;
      dense *dmat;
      sparse *smat;
      funct_def *funct;

These attributes are public

				FUNCTIONS :
				----------

constructors
destructor

operator =
output : outputs the value (doesn't print the type )   

===========================================================================*/



#ifndef VALUE_H
#define VALUE_H

#define NUM 256
#define DENSE 257
#define STRING 255
#define SPARSE 258
#define NO_TYPE 259
#define FUNCT 260

#include "dense.h"
#include "sparse.h"
#include "funct_def.h"

class value {

  public:
    // Constructors
    value ();
    value (char *);
    value (double);
    value (int);
    value (dense *);
    value (sparse *);
    value (funct_def *);
    value (const value &);

    // Destructor
    void free();

	char * get_str();
        double get_num();

    // Operator
    value operator = (const value &);

    int type;			// type of the value
    union { 			// value 
      char *str;
      double num;
      dense *dmat;
      sparse *smat;
      funct_def *funct;
    };
}; // end class value


extern int yyerror (char *);
void output (value *);

#endif
